@@ -3,17 +3,20 @@
 import datetime
 from functools import wraps
 from itertools import groupby
+
 from sql.conditionals import Case
+from sql.functions import CharLength
 
 from trytond.i18n import gettext
-from trytond.model import Workflow, ModelView, fields, ModelSQL
+from trytond.model import Index, ModelSQL, ModelView, Workflow, fields
 from trytond.modules.company import CompanyReport
+from trytond.modules.currency.fields import Monetary
 from trytond.modules.product import price_digits
 from trytond.pool import Pool, PoolMeta
-from trytond.pyson import Eval, Bool, If, Id
+from trytond.pyson import Bool, Eval, Id, If
 from trytond.tools import sortable_values
 from trytond.transaction import Transaction
-from trytond.wizard import Wizard, StateView, StateTransition, Button
+from trytond.wizard import Button, StateTransition, StateView, Wizard
 
 from .exceptions import PreviousQuotation
 
@@ -66,8 +69,7 @@ class ConfigurationSequence(metaclass=PoolMeta):
             ('sequence_type', '=',
                 Id('purchase_request_quotation',
                     'sequence_type_purchase_request_quotation')),
-            ],
-        depends=['company'])
+            ])
 
     @classmethod
     def default_purchase_request_quotation_sequence(cls):
@@ -90,15 +92,14 @@ class Quotation(Workflow, ModelSQL, ModelView):
         states={
             'required': ~Eval('state').in_(['draft', 'cancelled'])
             },
-        depends=['state'],
         help="The unique identifier of the quotation.")
     revision = fields.Integer('Revision', readonly=True,
         help="Number incremented each time the quotation is sent.")
-    reference = fields.Char('Reference', select=True,
+    reference = fields.Char(
+        "Reference",
         help="The reference used by the supplier.")
     company = fields.Many2One(
-        'company.company', "Company",
-        required=True, select=True,
+        'company.company', "Company", required=True,
         states={
             'readonly': Eval('state') != 'draft',
             })
@@ -112,28 +113,35 @@ class Quotation(Workflow, ModelSQL, ModelView):
         context={
             'company': Eval('company', -1),
             },
-        depends=['company'])
+        depends={'company'})
     supplier_address = fields.Many2One('party.address', 'Supplier Address',
         domain=[
             ('party', '=', Eval('supplier')),
-            ],
-        depends=['supplier'])
+            ])
     lines = fields.One2Many('purchase.request.quotation.line', 'quotation',
         'Lines', states={
             'readonly': Eval('state') != 'draft',
-            },
-        depends=['state'])
+            })
     state = fields.Selection([
         ('draft', 'Draft'),
         ('sent', 'Sent'),
-        ('rejected', 'Rejected'),
         ('received', 'Received'),
+        ('rejected', 'Rejected'),
         ('cancelled', 'Cancelled'),
-        ], 'State', readonly=True, required=True)
+        ], "State", readonly=True, required=True, sort=False)
 
     @classmethod
     def __setup__(cls):
+        cls.number.search_unaccented = False
+        cls.reference.search_unaccented = False
         super(Quotation, cls).__setup__()
+        t = cls.__table__()
+        cls._sql_indexes.update({
+                Index(t, (t.reference, Index.Similarity())),
+                Index(
+                    t, (t.state, Index.Equality()),
+                    where=t.state.in_(['draft', 'sent'])),
+                })
         cls._transitions |= set((
                 ('draft', 'cancelled'),
                 ('cancelled', 'draft'),
@@ -167,6 +175,11 @@ class Quotation(Workflow, ModelSQL, ModelView):
                     },
                 })
 
+    @classmethod
+    def order_number(cls, tables):
+        table, _ = tables[None]
+        return [CharLength(table.number), table.number]
+
     @classmethod
     def default_company(cls):
         return Transaction().context.get('company')
@@ -267,11 +280,8 @@ class QuotationLine(ModelSQL, ModelView):
     description = fields.Text('Description',
         states={
             'required': ~Eval('product')
-            },
-        depends=['product'])
-    quantity = fields.Float(
-        'Quantity', digits=(16, Eval('unit_digits', 2)), required=True,
-        depends=['unit_digits'])
+            })
+    quantity = fields.Float("Quantity", digits='unit', required=True)
     unit = fields.Many2One(
         'product.uom', 'Unit', ondelete='RESTRICT',
         states={
@@ -281,23 +291,20 @@ class QuotationLine(ModelSQL, ModelView):
             If(Bool(Eval('product_uom_category')),
                 ('category', '=', Eval('product_uom_category')),
                 ('category', '!=', -1)),
-        ],
-        depends=['product', 'product_uom_category'])
-    unit_digits = fields.Function(
-        fields.Integer('Unit Digits'), 'on_change_with_unit_digits')
+            ])
     product_uom_category = fields.Function(
-        fields.Many2One('product.uom.category', 'Product Uom Category'),
+        fields.Many2One(
+            'product.uom.category', "Product UoM Category",
+            help="The category of Unit of Measure for the product."),
         'on_change_with_product_uom_category')
-    unit_price = fields.Numeric('Unit Price', digits=price_digits)
+    unit_price = Monetary(
+        "Unit Price", currency='currency', digits=price_digits)
     currency = fields.Many2One('currency.currency', 'Currency',
         states={
             'required': Bool(Eval('unit_price')),
-            },
-        depends=['unit_price'])
-    currency_digits = fields.Function(
-        fields.Integer('Currency Digits'), 'on_change_with_currency_digits')
-    request = fields.Many2One('purchase.request', 'Request',
-        ondelete='CASCADE', select=True, required=True,
+            })
+    request = fields.Many2One(
+        'purchase.request', "Request", ondelete='CASCADE', required=True,
         domain=[
             If(Eval('quotation_state') == 'draft',
                 ('state', 'in', ['draft', 'quotation', 'received']), (), ),
@@ -305,14 +312,12 @@ class QuotationLine(ModelSQL, ModelView):
         states={
             'readonly': Eval('quotation_state') != 'draft'
             },
-        depends=['quotation_state'],
         help="The request which this line belongs to.")
     quotation = fields.Many2One('purchase.request.quotation', 'Quotation',
         ondelete='CASCADE', required=True,
         domain=[
                ('supplier', '=', Eval('supplier')),
-        ],
-        depends=['supplier'])
+        ])
     quotation_state = fields.Function(fields.Selection(
             'get_quotation_state', 'Quotation State'),
         'on_change_with_quotation_state', searcher='search_quotation_state')
@@ -343,34 +348,21 @@ class QuotationLine(ModelSQL, ModelView):
 
     @fields.depends('request',
         '_parent_request.product', '_parent_request.description',
-        '_parent_request.quantity', '_parent_request.uom',
+        '_parent_request.quantity', '_parent_request.unit',
         '_parent_request.company', '_parent_request.supply_date')
     def on_change_request(self):
         if self.request:
             self.product = self.request.product
             self.description = self.request.description
             self.quantity = self.request.quantity
-            self.unit = self.request.uom
+            self.unit = self.request.unit
             if self.request.company:
                 self.currency = self.request.company.currency
             self.supply_date = self.request.supply_date or datetime.date.max
 
-    @fields.depends('unit')
-    def on_change_with_unit_digits(self, name=None):
-        if self.unit:
-            return self.unit.digits
-        return None
-
     @fields.depends('product')
     def on_change_with_product_uom_category(self, name=None):
-        if self.product:
-            return self.product.default_uom_category.id
-
-    @fields.depends('currency')
-    def on_change_with_currency_digits(self, name=None):
-        if self.currency:
-            return self.currency.digits
-        return None
+        return self.product.default_uom_category if self.product else None
 
     @classmethod
     def get_quotation_state(cls):
@@ -440,7 +432,10 @@ class PurchaseRequestQuotationReport(CompanyReport):
         pool = Pool()
         Date = pool.get('ir.date')
         context = super().get_context(records, header, data)
-        context['today'] = Date.today()
+        company = header.get('company')
+        with Transaction().set_context(
+                company=company.id if company else None):
+            context['today'] = Date.today()
         return context
 
 
@@ -561,7 +556,7 @@ class CreatePurchaseRequestQuotation(Wizard):
         quotation_line.request = request
         quotation_line.description = request.description
         quotation_line.quantity = request.quantity
-        quotation_line.unit = request.uom
+        quotation_line.unit = request.unit
         quotation_line.currency = request.currency
         quotation_line.supply_date = request.supply_date or datetime.date.max
         return quotation_line
@@ -586,7 +581,6 @@ class PurchaseRequest(metaclass=PoolMeta):
             ('quotation_state', '=', 'received'),
             ('request', '=', Eval('id'))
         ],
-        depends=['id'],
         help="The quotation that will be chosen to create the purchase\n"
         "otherwise first ordered received quotation line will be selected.")
 
